"""
    plissken.cli
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~

    plissken's command line interface

"""

import os

import click


@click.group()
def main():
    """
    The main entry point for plissken
    :return:
    """
    pass


@main.command()
@click.argument("module_root")
@click.option("--out", type=str, default="docs")
@click.option("--format", type=str, default="md")
@click.option("--template_dir", type=str, default="")
@click.option("--template", type=str, default="")
def generate(module_root, out, format, template_dir, template):
    """
    This is a sub command on the main entry point group
    :return:
    """
    from plissken.documents.helpers import prettyDoc
    from plissken.documents.module import ModuleDocument
    from plissken.documents.package import PackageDoc, PackageDocument
    from plissken.file_operators import render_template

    def save_file(text, export_file_path):

        out_file = os.path.join(out, f"{export_file_path}.{format}")
        os.makedirs(os.path.dirname(out_file), exist_ok=True)
        with open(out_file, "w") as f:
            print(text, file=f)
        return

    def recursive_render(document):

        if isinstance(document, PackageDocument):
            render = render_template(f"{format}.template", data=prettyDoc(document))
            save_file(render, document.export_file_path)
            for package in document.sub_packages:
                return recursive_render(package)
            for module in document.modules:
                return recursive_render(module)
            return
        if isinstance(document, plissken.documents.module.ModuleDocument):
            render = render_template(f"{format}.template", data=prettyDoc(document))
            save_file(render, document.export_file_path)
            return

    package_docs = PackageDoc(module_root, package_root=os.path.basename(module_root))

    recursive_render(package_docs)

    pass



import typing
from collections import namedtuple

import redbaron

from plissken.documents.helpers import is_docstring

VariableDocument = namedtuple("VariableDocument", ["name", "annotation", "docstring"])


def VariableDoc(
    node: typing.Union[
        redbaron.nodes.NameNode,
        redbaron.nodes.AssignmentNode,
        redbaron.nodes.StandaloneAnnotationNode,
    ],
    doc: redbaron.nodes.StringNode,
):

    name = ""
    annotation = ""
    docstring = ""

    if isinstance(node, redbaron.nodes.NameNode):
        name = node.value
        try:
            annotation = node.annotation.value
        except:
            annotation = ""

    elif isinstance(node, redbaron.nodes.AssignmentNode):
        name = node.target.value
        try:
            annotation = node.annotation.value
        except:
            annotation = ""

    elif isinstance(node, redbaron.nodes.StandaloneAnnotationNode):
        name = node.target.value
        try:
            annotation = node.annotation.value
        except:
            annotation = ""
    else:
        raise ValueError(Node)

    docstring = is_docstring(doc)

    if not docstring:
        raise ValueError(
            f"The docstring for {self.name} doesn't appear valid. Got: {doc.value}"
        )
    return VariableDocument(name=name, annotation=annotation, docstring=docstring)



from collections import namedtuple

import redbaron

from plissken.documents.function import FunctionDoc
from plissken.documents.helpers import (
    get_documented_variables,
    get_init,
    is_docstring,
    method_is_class_method,
    method_is_dunder,
    method_is_private,
    method_is_semi_private,
)
from plissken.documents.variable import VariableDoc

ClassDocument = namedtuple(
    "ClassDocument",
    [
        "name",
        "docstring",
        "init_method",
        "dunder_methods",
        "semi_private_methods",
        "private_methods",
        "class_methods",
        "variables",
    ],
)


def ClassDoc(node: redbaron.nodes.ClassNode):

    name = node.name

    docstring = ""
    if isinstance(node[0], redbaron.nodes.StringNode):
        docstring = node[0]
    docstring = is_docstring(docstring)

    methods = [FunctionDoc(n) for n in node if isinstance(n, redbaron.nodes.DefNode)]

    init_method = get_init(methods)
    dunder_methods = [n for n in methods if method_is_dunder(n.name)]
    semi_private_methods = [n for n in methods if method_is_semi_private(n.name)]
    private_methods = [n for n in methods if method_is_private(n.name)]
    class_methods = [n for n in methods if method_is_class_method(n.name)]
    class_attributes = [VariableDoc(x[0], x[1]) for x in get_documented_variables(node)]
    variables = [VariableDoc(n[0], n[1]) for n in get_documented_variables(node)]

    return ClassDocument(
        name=name,
        docstring=docstring,
        init_method=init_method,
        dunder_methods=dunder_methods,
        semi_private_methods=semi_private_methods,
        private_methods=private_methods,
        class_methods=class_methods,
        variables=variables,
    )



import os

from jinja2 import ChoiceLoader, Environment, FileSystemLoader, select_autoescape
from redbaron import RedBaron


def code2red(filename: str) -> RedBaron:
    with open(filename) as sc:
        red = RedBaron(sc.read())
    return red


def render_template(template, data, directory=None) -> str:

    loader_list = [
        os.path.join(os.path.abspath(os.path.dirname(__file__)), "templates")
    ]
    if directory:
        if not isinstance(directory, list):
            directory = [directory]
        directory = [os.path.abspath(d) for d in directory]
        loader_list = directory + loader_list

    loader_list = [FileSystemLoader(d) for d in loader_list]

    env = Environment(
        loader=ChoiceLoader(loader_list),
        autoescape=select_autoescape(),
        extensions=["jinja2.ext.do"],
    )
    template = env.get_template(template)
    return template.render(data=data)



import os
import sys
from collections import namedtuple

import redbaron

from plissken.documents.function import FunctionDoc
from plissken.documents.helpers import get_documented_variables, is_docstring
from plissken.documents.klass import ClassDoc
from plissken.documents.variable import VariableDoc
from plissken.file_operators import code2red

ModuleDocument = namedtuple(
    "ModuleDocument",
    ["name", "docstring", "variables", "classes", "functions", "export_file_path"],
)


def get_qualified_name_from_path(file: str) -> str:

    base_name = os.path.basename(file)
    if base_name == "__init__.py":
        base_name = os.path.dirname(file)

    else:
        base_name = base_name[:-3]

    base_name = ".".join(os.path.split(base_name))

    if base_name.startswith("."):
        base_name = base_name[1:]

    return base_name


def split_all(path):
    allparts = []
    while 1:
        parts = os.path.split(path)
        if parts[0] == path:  # sentinel for absolute paths
            allparts.insert(0, parts[0])
            break
        elif parts[1] == path:  # sentinel for relative paths
            allparts.insert(0, parts[1])
            break
        else:
            path = parts[0]
            allparts.insert(0, parts[1])
    return allparts


def trim_prefix(path, package_root):
    rv = []
    path = split_all(path)
    while path:
        x = path.pop(0)
        if x == package_root:
            path.insert(0, x)
            return os.path.join(*path)


def ModuleDoc(file: str, package_root=None):
    # A Module is a python file that isn't an __init__.py file
    # It can have Variables, Classes, Methods, and/or a doc string

    node = code2red(file)

    if package_root:
        file = trim_prefix(file, package_root)

    export_file_path = os.path.splitext(file)[0]
    name = get_qualified_name_from_path(file)

    docstring = ""
    if isinstance(node[0], redbaron.nodes.StringNode):
        docstring = node[0]
    docstring = is_docstring(docstring)

    variables = [VariableDoc(n[0], n[1]) for n in get_documented_variables(node)]
    classes = [ClassDoc(n) for n in node if isinstance(n, redbaron.nodes.ClassNode)]
    functions = [FunctionDoc(n) for n in node if isinstance(n, redbaron.nodes.DefNode)]

    return ModuleDocument(
        name=name,
        docstring=docstring,
        variables=variables,
        classes=classes,
        functions=functions,
        export_file_path=export_file_path,
    )



import typing
from collections import namedtuple

import redbaron


def isnamedtupleinstance(x):
    t = type(x)
    b = t.__bases__
    if len(b) != 1 or b[0] != tuple:
        return False
    f = getattr(t, "_fields", None)
    if not isinstance(f, tuple):
        return False
    return all(type(n) == str for n in f)


def prettyDoc(document: namedtuple) -> str:

    rv = dict(document._asdict())
    for k, v in rv.items():
        if isnamedtupleinstance(v):
            rv[k] = prettyDoc(v)

        elif isinstance(v, list):
            rv[k] = [prettyDoc(x) for x in v]

        else:
            rv[k] = v
    return rv


def get_init(methods):
    init = [n for n in methods if method_is_init(n.name)]
    if len(init) > 1:
        raise ValueError("only one __init__ method allowed")

    rv_init = None
    if init:
        rv_init = init.pop()

    return rv_init


def get_documented_variables(node):
    rv = []
    for ix, n in enumerate(node):
        if isinstance(
            node[ix],
            (
                redbaron.nodes.StandaloneAnnotationNode,
                redbaron.nodes.AssignmentNode,
                redbaron.nodes.NameNode,
            ),
        ):
            try:
                if isinstance(node[ix + 1], redbaron.nodes.StringNode):
                    rv.append((node[ix], node[ix + 1]))
            except IndexError:
                pass

    return rv

    # return ClassDocument()


def method_is_init(name):
    return name == "__init__"


def method_is_semi_private(name):
    return name.startswith("_") and not name.startswith("__")


def method_is_private(name):
    return name.startswith("_") and not name.endswith("__")


def method_is_dunder(name):
    return name.startswith("__") and name.endswith("__") and name is not "__init__"


def method_is_class_method(name):
    return (
        not method_is_init(name)
        and not method_is_semi_private(name)
        and not method_is_private(name)
        and not method_is_dunder(name)
    )


def is_docstring(string: typing.Union[redbaron.nodes.StringNode, str]) -> str:

    if isinstance(string, redbaron.nodes.StringNode):
        string = string.value

    if string.startswith("'''") and string.endswith("'''"):
        return string[3:][:-3]

    if string.startswith('"""') and string.endswith('"""'):
        return string[3:][:-3]

    return ""



import os

from plissken.documents.helpers import prettyDoc
from plissken.documents.package import PackageDoc

__version__ = (
    open(os.path.join(os.path.dirname(__file__), "VERSION"), "r").read().strip()
)



import os
from collections import namedtuple

from plissken.documents.module import ModuleDoc

PackageDocument = namedtuple(
    "PackageDocument",
    [
        "name",
        "docstring",
        "variables",
        "classes",
        "functions",
        "sub_packages",
        "modules",
        "export_file_path",
    ],
)


def PackageDoc(directory: str, package_root=None):

    sub_packages = []
    modules = []

    if package_root is None:
        package_root = os.path.basename(directory)

    init_py_path = os.path.join(directory, "__init__.py")

    for root, dirs, filenames in os.walk(directory):
        for f in filenames:
            if f == "__init__.py":
                if root == directory:
                    package_doc = ModuleDoc(
                        os.path.join(root, f), package_root=package_root
                    )
            elif f.endswith(".py") and f is not "__init__.py":
                modules.append(os.path.join(root, f))

        for d in dirs:
            if os.path.isfile(os.path.join(root, d, "__init__.py")):
                sub_packages.append(os.path.join(root, d))

        break

    sub_packages = [PackageDoc(n, package_root=package_root) for n in sub_packages]
    modules = [ModuleDoc(n, package_root=package_root) for n in modules]
    name = package_doc.name
    docstring = package_doc.docstring
    variables = package_doc.variables
    classes = package_doc.classes
    functions = package_doc.functions
    export_file_path = package_doc.export_file_path.replace("__init__", "index")

    return PackageDocument(
        name=name,
        docstring=docstring,
        variables=variables,
        classes=classes,
        functions=functions,
        sub_packages=sub_packages,
        modules=modules,
        export_file_path=export_file_path,
    )



import os
import sys

import setuptools

VERSION = open(os.path.join("plissken", "VERSION")).read().strip()


requirements = ["click>7.0.0", "colander==1.7.0", "redbaron>=0.9.2", "jinja2 >= 2"]

dev_requirements = [
    "future",
    "pytest==3.8.0",
    "pytest-cov==2.6.0",
    "sphinx==1.8.0",
    "sphinx-rtd-theme==0.4.1",
    "angreal>=0.7.0",
    "mypy>=0.660",
    "lxml==4.3.0",
    "black==19.10b0",
    "pre-commit==2.2.0",
]


setuptools.setup(
    name="plissken",
    description="",
    long_description="""""",
    url="https://gitlab.com/dylanbstorey/plissken",
    author="dylanbstorey",
    author_email="dylan.storey@gmail.com",
    license="GPLv3",
    packages=setuptools.find_packages(),
    install_requires=requirements,
    zip_safe=False,
    version=VERSION,
    entry_points={"console_scripts": ["plissken=plissken.cli:main"]},
    python_requires=">=3",
    include_package_data=True,
    extras_require={"dev": dev_requirements},
)



from collections import namedtuple

import redbaron

from plissken.documents.helpers import is_docstring

ArgumentDocument = namedtuple("ArgumentDocument", ["name", "default", "annotation"])
DecoratorDocument = namedtuple("DecoratorDocument", ["name", "arguments"])
FunctionDocument = namedtuple(
    "FunctionDocument",
    [
        "name",
        "async_",
        "arguments",
        "docstring",
        "decorators",
        "code",
        "return_annotation",
    ],
)


def FunctionDoc(node: redbaron.nodes.DefNode) -> FunctionDocument:
    """
    A Function is a type of python construct within a Module.
    """
    doc_string = ""
    if isinstance(node[0], redbaron.nodes.StringNode):
        doc_string = node[0]

    decorators = []
    if node.decorators:
        decorators = _generate_decorators(node.decorators)

    return FunctionDocument(
        name=node.name,
        async_=node.async_ or False,
        arguments=_generate_arguments(node.arguments),
        docstring=is_docstring(doc_string),
        decorators=decorators,
        return_annotation=node.return_annotation,
        code=node.dumps(),
    )


def _generate_decorators(
    node_list: redbaron.base_nodes.DecoratorsLineProxyList
) -> list:
    decorators = []
    for decorator in node_list:

        assert isinstance(decorator, redbaron.nodes.DecoratorNode)

        call_arguments = []
        if decorator.call:
            for call in decorator.call:
                name = call.value.value

                value = ""
                if call.target:
                    value = call.target.value

                annotation = ""
                call_arguments.append(
                    ArgumentDocument(name=name, default=value, annotation=annotation)
                )

        name_list = decorator.value.value
        decorator_name = "@" + ".".join(
            [x.value for x in name_list if isinstance(x, redbaron.nodes.NameNode)]
        )

        decorators.append(
            DecoratorDocument(name=decorator_name, arguments=call_arguments)
        )
    return decorators


def _generate_arguments(node_list: redbaron.base_nodes.CommaProxyList) -> list:
    args = []
    for arg in node_list:
        if isinstance(arg, redbaron.nodes.DefArgumentNode):

            name = arg.target.value
            default = ""
            annotation = ""
            if arg.value:
                default = arg.value.value
            if arg.annotation:
                annotation = arg.annotation.value
            args.append(
                ArgumentDocument(name=name, default=default, annotation=annotation)
            )

        if isinstance(arg, redbaron.nodes.ListArgumentNode):
            args.append(
                ArgumentDocument(name=f"*{arg.value.value}", default="", annotation="")
            )

        if isinstance(arg, redbaron.nodes.DictArgumentNode):
            args.append(
                ArgumentDocument(name=f"**{arg.value.value}", default="", annotation="")
            )

    return args
